(function ($, Drupal, drupalSettings) {
  Drupal.behaviors.simple_interactive_maps = {
    attach: function (context, settings) {

      const maps = once('map-load', '.interactive_map__wrapper');
      const aspectRatio = 0.68;

      maps.forEach(async function (map_wrapper) {
        const mapId = map_wrapper.getAttribute('data-map-id');

        const map = map_wrapper.querySelector('.map-placeholder');

        const mapData = settings.simple_interactive_maps[mapId];

        let containerWidth = map.offsetWidth;
        let containerHeight = containerWidth * aspectRatio;
        let paper = Raphael(map, containerWidth, containerHeight);
        let paths = createElements(paper, mapData);
        let text = createText(paper, mapData);

        applyInteractions(paper, mapData, paths, text);
        fitMap(paths, paper, containerWidth, containerHeight);

        const textElems = map.querySelectorAll('text');
        textElems.forEach(function (textElem) {
          textElem.style.pointerEvents = 'none';
        });

        $(window).resize(function () {
          containerWidth = map.offsetWidth;
          containerHeight = containerWidth * aspectRatio;
          fitMap(paths, paper, containerWidth, containerHeight);
        });
      });
    },
  };

  function createElements(paper, mapData) {
    let paths = {};
    for (let state in mapData) {
      if (mapData[state]['hidden']) {
        continue;
      }
      let attr = {
        fill: mapData[state]['fill_color'],
        stroke: mapData[state]['stroke_color'],
        'stroke-width': 1,
        'stroke-linejoin': 'round',
        'data-label': mapData[state]['label'],
        'data-id': state,
      }
      if (mapData[state]['svg-data']['paths']) {
        mapData[state]['svg-data']['paths'].forEach(path => {
          paths[state] = paper.path(path).attr(attr);
          // Add an aria-label to the path.
          paths[state].node.setAttribute('aria-label', mapData[state]['label']);
        });
      }
    }
    return paths;
  }

  function createText(paper, mapData) {
    let attr = {
      'font-size': '11px',
      'font-family': '"Rubik","Source Sans Pro Web",sans-serif',
      'fill': '#e6e6e6'
    }

    let text = {};
    for (let state in mapData) {
      if (mapData[state]['hidden'] || !mapData[state]['text']) {
        continue;
      }
      attr['fill'] = mapData[state]['text_color'];
      text[state] = paper.text(mapData[state].text.x, mapData[state].text.y, mapData[state].text.text).attr(attr);
    }
    return text;
  }

  function applyInteractions(paper, mapData, paths, text) {
    for (let region in paths) {

      // Skip hidden states.
      if (mapData[region]['hidden']) {
        continue;
      }

      // Set up tooltip if there is content to be displayed.
      if (mapData[region]['tooltip']['processed'] != '') {
        new Opentip(paths[region][0], mapData[region]['tooltip']['processed']);
      }

      // Set up mouseover and mouseout color events.
      let pathOnMouseOver = function (event) {
        paths[region].animate({ fill: mapData[region]['hover_color'] }, 500);
      };

      let textOnMouseOver = function (event) {
        text[region].attr({ 'font-weight': 'bold' });
      }

      let pathOnMouseOut = function () {
        paths[region].animate({ fill: mapData[region]['fill_color'] }, 500);
      };

      let textOnMouseOut = function () {
        text[region].attr({ 'font-weight': 'normal' });
      }

      if (paths[region]) {
        paths[region][0].style.cursor = 'pointer';
        paths[region][0].onmouseover = pathOnMouseOver;
        paths[region][0].onmouseout = pathOnMouseOut;
        Drupal.simple_interactive_maps[mapData[region]['action']](paths[region][0], mapData[region]['action_configuration'], mapData[region]);
      }
      if (text[region]) {
        text[region][0].style.cursor = 'pointer';
        text[region][0].onmouseover = textOnMouseOver;
        text[region][0].onmouseout = textOnMouseOut;
        Drupal.simple_interactive_maps[mapData[region]['action']](text[region][0], mapData[region]['action_configuration'], mapData[region]);
      }
      if (mapData[region]['action'] === 'none') {

        continue;
      }
      // Call a dynamic function based on the action type.

      // if (mapData[region]['action'] === 'content') {
      //   paths[region][0].style.cursor = 'pointer';
      //   text[region][0].style.cursor = 'pointer';
      //
      //   paths[region][0].onclick = function (event) {
      //     // Create a new Drupal dialog and set its content.
      //     let mapDetailDialog = Drupal.dialog('<div>' + mapData[region]['content']['processed'] + '</div>', {
      //       title: mapData[region]['label'],
      //       dialogClass: 'interactive-map-dialog',
      //       autoOpen: true,
      //       resizable: false,
      //       modal: true,
      //       width: 800,
      //       closeOnEscape: true,
      //     });
      //     // Open the dialog.
      //     mapDetailDialog.showModal();
      //   };
      //
      //   text[region][0].onclick = function (event) {
      //     // Create a new Drupal dialog and set its content.
      //     let mapDetailDialog = Drupal.dialog('<div>' + mapData[region]['content']['processed'] + '</div>', {
      //       title: mapData[region]['label'],
      //       dialogClass: 'interactive-map-dialog',
      //       autoOpen: true,
      //       resizable: false,
      //       modal: true,
      //       width: 800,
      //       closeOnEscape: true,
      //     });
      //     // Open the dialog.
      //     mapDetailDialog.showModal();
      //   }
      // }

    }
  }

  function fitMap(paths, paper, width, height) {
    let st = paper.set();
    for (let statePath in paths) {
      st.push(paths[statePath]);
    }
    let bbox = st.getBBox();
    bbox = isFinite(bbox.width) && isFinite(bbox.height) ? bbox : { x: 0, y: 0, width: 1000, height: 900 };
    paper.setViewBox(bbox.x, bbox.y, bbox.width, bbox.height, true);
    paper.setSize(width, height);
  }

})(jQuery, Drupal, drupalSettings);

var mapData;

Opentip.styles.simple_interactive_maps = {
  extends: 'standard',
  stem: true,
  fixed: true,
  tipJoint: 'bottom',
  showOn: 'mouseover',
  hideOn: 'mouseout',
  removeElementsOnHide: true,
  containInViewport: true,
};
Opentip.defaultStyle = 'simple_interactive_maps';

Drupal.simple_interactive_maps = Drupal.simple_interactive_maps || {};
