<?php

namespace Drupal\simple_interactive_maps;

/**
 * Helper class for working with SVG paths.
 */
class SvgHelper {

  /**
   * Get the bounding box of an SVG path.
   *
   * @param string $path
   *   The SVG path.
   *
   * @return array
   *   An array containing the x, y, width, and height of the bounding box.
   */
  public static function getBoundingBox($path): array {
    $commands = preg_split('/(?=[MmZzLlHhVvCcSsQqTtAa])/', $path, -1, PREG_SPLIT_NO_EMPTY);

    $minX = INF;
    $maxX = -INF;
    $minY = INF;
    $maxY = -INF;

    $currentX = 0;
    $currentY = 0;

    foreach ($commands as $command) {
      $type = $command[0];
      $args = preg_split('/[\s,]+/', substr($command, 1));

      switch ($type) {
        case 'M':
        case 'L':
          for ($i = 0; $i < count($args); $i += 2) {
            $currentX = $args[$i];
            $currentY = $args[$i + 1];
            $minX = min($minX, $currentX);
            $maxX = max($maxX, $currentX);
            $minY = min($minY, $currentY);
            $maxY = max($maxY, $currentY);
          }
          break;

        case 'H':
          foreach ($args as $x) {
            $currentX = $x;
            $minX = min($minX, $currentX);
            $maxX = max($maxX, $currentX);
          }
          break;

        case 'V':
          foreach ($args as $y) {
            $currentY = $y;
            $minY = min($minY, $currentY);
            $maxY = max($maxY, $currentY);
          }
          break;

      }
    }

    return [$minX, $minY, $maxX - $minX, $maxY - $minY];
  }

  /**
   * Calculate the viewBox attribute for an SVG element containing multiple paths.
   *
   * @param array $paths
   *   An array of SVG paths.
   *
   * @return string
   *   The viewBox attribute value.
   */
  public static function calculateViewBox($paths): string {
    $minX = INF;
    $maxX = -INF;
    $minY = INF;
    $maxY = -INF;

    foreach ($paths as $path) {
      [$x, $y, $width, $height] = self::getBoundingBox($path);
      $minX = min($minX, $x);
      $maxX = max($maxX, $x + $width);
      $minY = min($minY, $y);
      $maxY = max($maxY, $y + $height);
    }

    return "$minX $minY " . ($maxX - $minX) . " " . ($maxY - $minY);
  }

}
